/**
 * @fileoverview Check for dependent keys with invalid `@each` usage
 * @author onechiporenko
 * @copyright 2016 onechiporenko. All rights reserved.
 * See LICENSE file in root directory for full license.
 */
"use strict";
var ember = require("../utils/ember.js");
//------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------

module.exports = function(context) {

  var m2 = "Multiple `@each` in the one dependent key are not allowed.";
  var m3 = "Deep `@each` in the dependent key is not allowed.";

  function onlyUnique(value, index, self) {
    return self.indexOf(value) === index;
  }

  //--------------------------------------------------------------------------
  // Public
  //--------------------------------------------------------------------------

  return {
    "CallExpression": function (node) {
      if (ember.isEmberField(node)) {
        var dependentKeys = ember.getDependentKeys(node);
        dependentKeys = ember.expandDependentKeys(dependentKeys.filter(onlyUnique));
        var report2 = false;
        var report3 = false;
        for (var i = 0; i < dependentKeys.length; i++) {
          if (dependentKeys[i].indexOf("@each") === -1) {
            continue;
          }
          var chunks = dependentKeys[i].split("@each");
          if (chunks.length > 2) {
            report2 = true;
            continue;
          }
          if (chunks[1].split(".").length > 2) {
            report3 = true;
          }
        }
        if (report2) {
          context.report(node, m2);
        }
        if (report3) {
          context.report(node, m3);
        }
      }
    }
  };

};

module.exports.schema = [];